@@ -24,12 +24,6 @@
 
 ;;; Code:
 
-(defalias 'riece-set-case-syntax-pair
-  'set-case-syntax-pair)
-
-(defalias 'riece-copy-case-table
-  'copy-case-table)
-
 ;;; stolen (and renamed) from gnus-ems.el.
 ;;; In GNU Emacs, user can intercept whole mouse tracking events by
 ;;; assigning [mouse-X].  In XEmacs, however, which causes different
@@ -95,6 +89,12 @@
 (defalias 'riece-run-with-idle-timer 'run-with-idle-timer)
 (defalias 'riece-cancel-timer 'cancel-timer)
 
+(defalias 'riece-match-string-no-properties 'match-string-no-properties)
+
+(defun riece-propertize-modeline-string (string &rest properties)
+  (add-text-properties string properties)
+  string)
+
 (provide 'riece-emacs)
 
 ;;; riece-emacs.el ends here
