@@ -28,7 +28,6 @@
 (require 'riece-misc)
 (require 'riece-server)			;riece-close-server
 (require 'riece-identity)
-(require 'riece-display)
 
 (defun riece-handle-numeric-reply (prefix number name string)
   (let ((base-number (* (/ number 100) 100))
@@ -158,7 +157,7 @@
       (riece-redisplay-buffers)
       (riece-close-server-process process)
       ;; If no server process is available, exit.
-      (unless riece-process-list
+      (unless riece-server-process-alist
 	(riece-exit)))))
 
 (provide 'riece-filter)
