@@ -27,26 +27,27 @@
 (require 'riece-globals)
 (require 'riece-channel)
 (require 'riece-user)
-(require 'riece-display)
+(require 'riece-signal)
 
 (defun riece-naming-assert-join (user-name channel-name)
   (riece-user-toggle-channel user-name channel-name t)
   (riece-channel-toggle-user channel-name user-name t)
-  (if (riece-identity-equal-no-server user-name riece-real-nickname)
-      (let ((channel-identity (riece-make-identity channel-name
-						   riece-server-name)))
-	(riece-join-channel channel-identity)
-	(riece-switch-to-channel channel-identity)
-	(setq riece-join-channel-candidate nil))))
+  (riece-emit-signal 'user-joined-channel
+		     (riece-make-identity user-name
+					  riece-server-name)
+		     (riece-make-identity channel-name
+					  riece-server-name)))
 
 (defun riece-naming-assert-part (user-name channel-name)
   (riece-user-toggle-channel user-name channel-name nil)
   (riece-channel-toggle-user channel-name user-name nil)
   (riece-channel-toggle-operator channel-name user-name nil)
   (riece-channel-toggle-speaker channel-name user-name nil)
-  (if (riece-identity-equal-no-server user-name riece-real-nickname)
-      (riece-part-channel (riece-make-identity channel-name
-					       riece-server-name))))
+  (riece-emit-signal 'user-left-channel
+		     (riece-make-identity user-name
+					  riece-server-name)
+		     (riece-make-identity channel-name
+					  riece-server-name)))
 
 (defun riece-naming-assert-rename (old-name new-name)
   (if (riece-identity-equal-no-server old-name riece-real-nickname)
@@ -57,26 +58,35 @@
 	 users user)
     (while channels
       (setq users (riece-channel-get-users (car channels))
-	    user (car (riece-identity-assoc old-name users t)))
+	    user (riece-identity-assoc old-name users t))
       (if user
 	  (setcar user new-name))
       (setq channels (cdr channels)))
     (riece-rename-user old-name new-name))
-  ;; Rename the channel buffer.
-  (let* ((old-identity (riece-make-identity old-name riece-server-name))
-	 (new-identity (riece-make-identity new-name riece-server-name))
-	 (pointer (riece-identity-member old-identity riece-current-channels)))
-    (when pointer
-      (setcar pointer new-identity)
-      (with-current-buffer (riece-channel-buffer old-identity)
-	(rename-buffer (riece-channel-buffer-name new-identity) t)
-	(setq riece-channel-buffer-alist
-	      (cons (cons new-identity (current-buffer))
-		    (delq (riece-identity-assoc old-identity
-						riece-channel-buffer-alist)
-			  riece-channel-buffer-alist))))
-      (if (riece-identity-equal old-identity riece-current-channel)
-	  (riece-switch-to-channel new-identity)))))
+  (riece-emit-signal 'user-renamed
+		     (riece-make-identity old-name riece-server-name)
+		     (riece-make-identity new-name riece-server-name)))
+
+(defun riece-naming-assert-channel-users (users channel-name)
+  (let ((channel-identity (riece-make-identity channel-name
+					       riece-server-name))
+	user-identity-list)
+    (while users
+      (riece-user-toggle-channel (car (car users)) channel-name t)
+      (riece-channel-toggle-user channel-name (car (car users)) t)
+      (if (memq ?o (cdr (car users)))
+	  (riece-channel-toggle-operator channel-name (car (car users)) t)
+	(if (memq ?v (cdr (car users)))
+	    (riece-channel-toggle-speaker channel-name (car (car users)) t)
+	  (riece-channel-toggle-operator channel-name (car (car users)) nil)
+	  (riece-channel-toggle-speaker channel-name (car (car users)) nil)))
+      (setq user-identity-list
+	    (cons (cons (riece-make-identity (car (car users))
+					     riece-server-name)
+			(cdr (car users)))
+		  user-identity-list)
+	    users (cdr users)))
+    (riece-emit-signal 'user-list-changed channel-identity)))
 
 (provide 'riece-naming)
 
